/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global data*/
/*exported watch*/

var watch = (function() {
    var CIRCLE_RADIUS_CENTER = 6,
        CIRCLE_RADIUS_CITY = 2.5,
        CIRCLE_RADIUS_AROUND = 1.5,
        NEEDLE_LENGTH_HOUR = 0.65,
        NEEDLE_LENGTH_MINUTE = 0.8,
        NEEDLE_LENGTH_SECOND = 0.8,
        NEEDLE_WIDTH_HOUR = 4,
        NEEDLE_WIDTH_MINUTE = 2,
        NEEDLE_WIDTH_SECOND = 2,
        LOCATION_AROUND_CIRCLE = 0.9,
        LOCATION_AROUND_LINE_START = 0.82,
        LOCATION_AROUND_LINE_END = 0.92,
        centerX = document.body.clientWidth / 2,
        centerY = document.body.clientHeight / 2,
        watchRadius = document.body.clientWidth / 2,
        canvas = null,
        ctx = null,
        watch = {};

    /**
     * Renders a circle with specific center, radius, and color
     * @private
     * @param {object} context - the context of the canvas for the circle to be placed in
     * @param {number} radius - the radius of the circle
     * @param {string} color - the color of the circle
     */
    function renderCircle(context, centerX, centerY, radius, color) {
        context.save();
        context.beginPath();
        context.fillStyle = color;
        context.arc(centerX, centerY, radius, 0, 2 * Math.PI);
        context.fill();
        context.closePath();
        context.restore();
    }

    /**
     * Renders a needle with specific center, angle, start point, end point, width and color
     * @private
     * @param {object} context - the context of the canvas for the needle to be placed in
     * @param {number} angle - the angle of the needle (0 ~ 360)
     * @param {number} startPoint - the start point of the needle (-1.0 ~ 1.0)
     * @param {number} endPoint - the end point of the needle (-1.0 ~ 1.0)
     * @param {number} width - the width of the needle
     * @param {string} color - the color of the needle
     */
    function renderNeedle(context, angle, startPoint, endPoint, width, color) {
        var radius = context.canvas.width / 2,
            centerX = context.canvas.width / 2,
            centerY = context.canvas.height / 2,
            dxi = radius * Math.cos(angle) * startPoint,
            dyi = radius * Math.sin(angle) * startPoint,
            dxf = radius * Math.cos(angle) * endPoint,
            dyf = radius * Math.sin(angle) * endPoint;

        context.save();
        context.beginPath();
        context.lineWidth = width;
        context.strokeStyle = color;
        context.moveTo(centerX + dxi, centerY + dyi);
        context.lineTo(centerX + dxf, centerY + dyf);
        context.stroke();
        context.closePath();
        context.restore();
    }

    /**
     * Renders an image on the canvas
     * @private
     * @param {object} context - the context of the canvas for the image to be placed in
     * @param {string} imgPath - the path of the image
     */
    function renderImg(context, imgPath) {
        var map = new Image(),
            x = 0,
            y = ((document.body.clientHeight / 2) - (document.body.clientWidth / 2));

        map.src = imgPath;
        context.drawImage(map, x, y, document.body.clientWidth, document.body.clientWidth);
    }

    /**
     * Draws the background layout of the watch
     * @private
     */
    function drawWatchLayout() {
        var i,
            dx,
            dy,
            angle = 0;

        ctx.clearRect(0, 0, ctx.canvas.width, ctx.canvas.height);

        // Draw background map image
        renderImg(ctx, "./image/map.png");

        // 12 unit divider
        for (i = 1; i <= 12; i++) {
            angle = (i - 3) * (Math.PI * 2) / 12;
            if (i === 12) {
                renderNeedle(ctx, angle, LOCATION_AROUND_LINE_START, LOCATION_AROUND_LINE_END, 3, "#454545");
            } else {
                dx = centerX + (watchRadius * LOCATION_AROUND_CIRCLE) * Math.cos(angle);
                dy = centerY + (watchRadius * LOCATION_AROUND_CIRCLE) * Math.sin(angle);
                renderCircle(ctx, dx, dy, CIRCLE_RADIUS_AROUND, "#454545");
            }
        }
    }

    /**
     * Draws the content of the watch for the current (default) city
     * @private
     */
    function drawWatchContentDefault(hour, minute, second) {
        var hourNeedleAngle = Math.PI * (((hour + minute / 60) / 6) - 0.5),
            minuteNeedleAngle = Math.PI * (((minute + second / 60) / 30) - 0.5),
            secondNeedleAngle = Math.PI * ((second / 30) - 0.5);

        ctx.shadowBlur = 5;
        ctx.shadowOffsetX = 1;
        ctx.shadowOffsetY = 1;

        // Draw the hour needle
        renderNeedle(ctx, hourNeedleAngle, 0, NEEDLE_LENGTH_HOUR, NEEDLE_WIDTH_HOUR, "#454545");

        // Draw the minute needle
        renderNeedle(ctx, minuteNeedleAngle, 0, NEEDLE_LENGTH_MINUTE, NEEDLE_WIDTH_MINUTE, "#454545");

        // Draw the second needle
        renderNeedle(ctx, secondNeedleAngle, 0, NEEDLE_LENGTH_SECOND, NEEDLE_WIDTH_SECOND, "#c7c7c7");

        // Draw the center circle
        renderCircle(ctx, centerX, centerY, CIRCLE_RADIUS_CENTER, "#454545");
        ctx.shadowBlur = 0;
        ctx.shadowOffsetX = 0;
        ctx.shadowOffsetY = 0;
    }

    /**
     * Draws the content of the watch for a specific city
     * @param {number} cityIndex - the index of the specific city in colorCity array
     * @private
     */
    function drawWatchContentCity(cityIndex) {
        var hour = tizen.time.getCurrentDateTime().toTimezone(data.cityArray[data.colorCity[cityIndex].cityId].tzid).getHours(),
            minute = tizen.time.getCurrentDateTime().toTimezone(data.cityArray[data.colorCity[cityIndex].cityId].tzid).getMinutes(),
            color = data.colorCity[cityIndex].colorCode,
            mapX = data.cityArray[data.colorCity[cityIndex].cityId].mapX * (document.body.clientWidth / 360),
            mapY = data.cityArray[data.colorCity[cityIndex].cityId].mapY * (document.body.clientWidth / 360),
            hourNeedleAngle = Math.PI * (((hour + minute / 60) / 6) - 0.5);

        // Draw the marker on the map
        renderCircle(ctx, centerX + mapX, centerY + mapY, CIRCLE_RADIUS_CITY, color);
        ctx.shadowBlur = 5;
        ctx.shadowOffsetX = 1;
        ctx.shadowOffsetY = 1;

        // Draw the hour needle
        ctx.shadowColor = "#454545";
        ctx.shadowBlur = 10;
        ctx.shadowOffsetX = 2;
        ctx.shadowOffsetY = 2;

        renderNeedle(ctx, hourNeedleAngle, 0, NEEDLE_LENGTH_HOUR, NEEDLE_WIDTH_HOUR, color);

        ctx.shadowBlur = 0;
        ctx.shadowOffsetX = 0;
        ctx.shadowOffsetY = 0;

        // Draw the center circle
        renderCircle(ctx, centerX, centerY, CIRCLE_RADIUS_CENTER, "#454545");
        ctx.shadowBlur = 0;
        ctx.shadowOffsetX = 0;
        ctx.shadowOffsetY = 0;
    }

    /**
     * Draws the whole watch
     * @public
     */
    watch.drawWatch = function() {
        var i,
            date = tizen.time.getCurrentDateTime(),
            hour = date.getHours(),
            minute = date.getMinutes(),
            second = date.getSeconds();

        drawWatchLayout();
        drawWatchContentDefault(hour, minute, second);

        for (i = 0; i < data.colorCity.length; i++) {
            if (data.cityArray[data.colorCity[i].cityId].tzid !== "NONE") {
                drawWatchContentCity(i);
            }
        }
    };

    /**
     * Sets default variables for drawing watch
     * @public
     */
    watch.setDefaultVariables = function() {
        canvas = document.querySelector("#canvas-watch");
        ctx = canvas.getContext("2d");
        canvas.width = document.body.clientWidth;
        canvas.height = document.body.clientHeight;
    };

    return watch;
}());